#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

#region Global Enums

public enum anaHiLoActivatorBaseTypeU1 {Median, ADXVMA, Butterworth_2, Butterworth_3, DEMA, DSMA, DTMA, DWMA, Ehlers, EMA, Gauss_2, Gauss_3, Gauss_4, 
			HMA, HoltEMA, LinReg, SMA, SuperSmoother_2, SuperSmoother_3, TEMA, TMA, TSMA, TWMA, VWMA, WMA, ZeroLagHATEMA, ZeroLagTEMA, ZLEMA}

#endregion

namespace NinjaTrader.Indicator
{
    /// <summary>
    /// 
    /// </summary>
    [Description("")]
    [Gui.Design.DisplayName("anaHiLoActivatorU1")]
    public class anaHiLoActivatorU1 : Indicator
    {
        #region Variables
            private int 						period 				= 50; 
			private double						currentStopLong		= 0.0;
			private double						currentStopShort	= 0.0;
			private double						priorStopLong		= 0.0;
			private double						priorStopShort		= 0.0;
			private double						margin				= 0.0;
			private anaHiLoActivatorBaseTypeU1 	thisBaseType		= anaHiLoActivatorBaseTypeU1.SMA; 
			private bool						candles				= false;
			private bool 						gap					= false;
			private bool 						showArrows 			= true;
			private bool 						showPaintBars 		= false;
			private bool 						showStopLine		= true;
			private bool						soundAlert			= false;
			private bool						oneTickRepaint		= true;
			private bool 						currentUpTrend		= true;
			private bool 						priorUpTrend		= true;
			private bool						repaint				= false;
			private int							opacity				= 3;
			private int							alpha				= 0;
			private int 						plot0Width 			= 1;
			private PlotStyle 					plot0Style			= PlotStyle.Dot;
			private DashStyle					dash0Style			= DashStyle.Dot;
			private int 						plot1Width 			= 1;
			private PlotStyle 					plot1Style			= PlotStyle.Line;
			private DashStyle 					dash1Style			= DashStyle.Solid;
			private Color 						upColor				= Color.DodgerBlue;
			private Color 						downColor			= Color.Red;
			private int 						rearmTime			= 30;
			private string 						confirmedUpTrend	= "newuptrend.wav";
			private string 						confirmedDownTrend	= "newdowntrend.wav";
			private string 						potentialUpTrend	= "potentialuptrend.wav";
			private string 						potentialDownTrend	= "potentialdowntrend.wav";
			private BoolSeries 					upTrend;
 			private IDataSeries highAverage;
			private IDataSeries lowAverage;
 			private ATR							volatility;
      #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			Add(new Plot(Color.Gray, PlotStyle.Dot, "StopDot"));
            Add(new Plot(Color.Gray, PlotStyle.Line, "StopLine"));
            CalculateOnBarClose	= true;
            Overlay				= true;
			PlotsConfigurable 	= false;
			upTrend 			= new BoolSeries(this);
        }

 		protected override void OnStartUp()
		{
			switch (thisBaseType)
			{
				case anaHiLoActivatorBaseTypeU1.Median: 
					highAverage = anaMovingMedian(High, period);
					lowAverage = anaMovingMedian(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.ADXVMA: 
					highAverage = anaADXVMA(High, period);
					lowAverage = anaADXVMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.Butterworth_2: 
					highAverage = anaButterworthFilter(High, period, 2);
					lowAverage = anaButterworthFilter(Low, period, 2);
					break;
				case anaHiLoActivatorBaseTypeU1.Butterworth_3: 
					highAverage = anaButterworthFilter(High, period, 3);
					lowAverage = anaButterworthFilter(Low, period, 3);
					break;
				case anaHiLoActivatorBaseTypeU1.DEMA: 
					highAverage = DEMA(High, period);
					lowAverage = DEMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.DSMA: 
					highAverage = DSMA(High, period);
					lowAverage = DSMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.DTMA: 
					highAverage = DTMA(High, period);
					lowAverage = DTMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.DWMA: 
					highAverage = DWMA(High, period);
					lowAverage = DWMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.Ehlers: 
					highAverage = anaEhlersFilter(High, period);
					lowAverage = anaEhlersFilter(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.EMA: 
					highAverage = EMA(High, period);
					lowAverage = EMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.Gauss_2: 
					highAverage = anaGaussianFilter(High, period, 2);
					lowAverage = anaGaussianFilter(Low, period, 2);
					break;
				case anaHiLoActivatorBaseTypeU1.Gauss_3: 
					highAverage = anaGaussianFilter(High, period, 3);
					lowAverage = anaGaussianFilter(Low, period, 3);
					break;
				case anaHiLoActivatorBaseTypeU1.Gauss_4: 
					highAverage = anaGaussianFilter(High, period, 4);
					lowAverage = anaGaussianFilter(Low, period, 4);
					break;
				case anaHiLoActivatorBaseTypeU1.HMA: 
					highAverage = HMA(High, period);
					lowAverage = HMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.HoltEMA: 
					highAverage = anaHoltEMA(High, period, 2*period);
					lowAverage = anaHoltEMA(Low, period, 2*period);
					break;
				case anaHiLoActivatorBaseTypeU1.LinReg: 
					highAverage = LinReg(High, period);
					lowAverage = LinReg(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.SMA: 
					highAverage = SMA(High, period);
					lowAverage = SMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.SuperSmoother_2: 
					highAverage = anaSuperSmootherFilter(High, period, 2);
					lowAverage = anaSuperSmootherFilter(Low, period, 2);
					break;
				case anaHiLoActivatorBaseTypeU1.SuperSmoother_3: 
					highAverage = anaSuperSmootherFilter(High, period, 3);
					lowAverage = anaSuperSmootherFilter(Low, period, 3);
					break;
				case anaHiLoActivatorBaseTypeU1.TEMA: 
					highAverage = TEMA(High, period);
					lowAverage = TEMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.TMA: 
					highAverage = TMA(High, period);
					lowAverage = TMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.TSMA: 
					highAverage = TSMA(High, period);
					lowAverage = TSMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.TWMA: 
					highAverage = TWMA(High, period);
					lowAverage = TWMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.VWMA: 
					highAverage = VWMA(High, period);
					lowAverage = VWMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.WMA: 
					highAverage = WMA(High, period);
					lowAverage = WMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.ZeroLagHATEMA: 
					highAverage = anaZeroLagHATEMA(High, period);
					lowAverage = anaZeroLagHATEMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.ZeroLagTEMA: 
					highAverage = ZeroLagTEMA(High, period);
					lowAverage = ZeroLagTEMA(Low, period);
					break;
				case anaHiLoActivatorBaseTypeU1.ZLEMA: 
					highAverage = ZLEMA(High, period);
					lowAverage = ZLEMA(Low, period);
					break;
			}
			volatility = ATR(Close, 256); 
			Plots[0].Pen.Width = plot0Width;
			Plots[0].PlotStyle = plot0Style;
			Plots[0].Pen.DashStyle = dash0Style;
			Plots[1].Pen.Width = plot1Width;
			Plots[1].PlotStyle = plot1Style;
			Plots[1].Pen.DashStyle = dash1Style;
			if (ChartControl != null && ChartControl.ChartStyleType == ChartStyleType.CandleStick)
			{
				candles = true;
				alpha = 25*opacity;
			}
			else
				candles = false;
			if(CalculateOnBarClose)
				oneTickRepaint = false;
			if(ChartControl != null && oneTickRepaint)
				repaint = true;
			else
				repaint = false;
			gap = (plot1Style == PlotStyle.Line)||(plot1Style == PlotStyle.Square);
		}
		
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if(CurrentBar < period)
			{
				if(CurrentBar == period - 1)
				{
					upTrend.Set(true);
					StopDot.Set (lowAverage[1]);
					StopLine.Set (lowAverage[1]); 
				}
				return;
			}
			
			if(FirstTickOfBar)
			{
				priorUpTrend = currentUpTrend;
				priorStopLong = currentStopLong;
				priorStopShort = currentStopShort;
				currentStopShort = highAverage[1];
				currentStopLong = lowAverage[1];
				margin = volatility[1];
			}
			
			if (Historical || CalculateOnBarClose) // the trend is determined with the last and only tick of the current bar
			{
				if (Close[0] > currentStopShort)
					currentUpTrend = true;
				else if (Close[0] < currentStopLong)
					currentUpTrend = false;
			}
			else if (FirstTickOfBar) // the trend can only be determined with the first tick of the next bar 		
			{
				if (Close[1] > priorStopShort)
					currentUpTrend = true;
				else if (Close[1] < priorStopLong)  
					currentUpTrend = false;
			}	
			
			if(FirstTickOfBar)
			{
				if(repaint && !Historical)
				{
					upTrend.Set(1, currentUpTrend);
					if(currentUpTrend)
					{
						StopDot.Set(1, priorStopLong);
						PlotColors[0][1] = upColor;
						if(showStopLine)
						{
							StopLine.Set(1, priorStopLong);
							if(gap && !priorUpTrend)
								PlotColors[1][1]= Color.Transparent;
							else
								PlotColors[1][1] = upColor;
						}
					}
					else
					{
						StopDot.Set(1, priorStopShort);
						PlotColors[0][1] = downColor;
						if(showStopLine)
						{
							StopLine.Set(1, priorStopShort);
							if(gap && priorUpTrend)
								PlotColors[1][1]= Color.Transparent;
							else
								PlotColors[1][1] = downColor;
						}
					}
				}
				upTrend.Set(currentUpTrend);
				if(currentUpTrend)
				{
					StopDot.Set(currentStopLong);
					PlotColors[0][0] = upColor;
					if(showStopLine)
					{
						StopLine.Set(currentStopLong);
						if(gap && !priorUpTrend)
							PlotColors[1][0]= Color.Transparent;
						else
							PlotColors[1][0] = upColor;
					}
					else
						StopLine.Reset();
				}
				else
				{
					StopDot.Set(currentStopShort);
					PlotColors[0][0] = downColor;
					if(showStopLine)
					{
						StopLine.Set(currentStopShort);
						if(gap && priorUpTrend)
							PlotColors[1][0]= Color.Transparent;
						else
							PlotColors[1][0] = downColor;
					}
					else
						StopLine.Reset();
				}
			
				if(ShowPaintBars)
				{
					if(repaint && !Historical)
					{
						if (currentUpTrend)
						{
							CandleOutlineColorSeries[1] = upColor;
							BarColorSeries[1] = upColor;
						}
						else
						{
							CandleOutlineColorSeries[1] = downColor;
							BarColorSeries[1]  = downColor;
						}
						if(Open[1] < Close[1] && candles) 
							BarColorSeries[1] = Color.FromArgb(alpha, BarColor);
					}
					if (currentUpTrend)
						CandleOutlineColor = upColor;
					else
						CandleOutlineColor = downColor;
					BarColor = CandleOutlineColor;
				}
				
				if(ShowArrows)
				{
					int barsBack = 0;
					double low = currentStopLong;
					double high = currentStopShort;
					if(repaint && !Historical)
					{
						barsBack = 1;
						low = priorStopLong;
						high = priorStopShort;
					}
					if(currentUpTrend && !priorUpTrend)
						DrawArrowUp("arrow" + CurrentBar, true, barsBack, low - 0.25 * margin, upColor);
					else if(!currentUpTrend && priorUpTrend)
						DrawArrowDown("arrow" + CurrentBar, true, barsBack, high + 0.25 * margin, downColor);
				}
				
				if (soundAlert && !Historical && IsConnected() && CalculateOnBarClose)
				{
					if(currentUpTrend && !priorUpTrend)		
					{
						try
						{
								Alert("NewUpTrend", NinjaTrader.Cbi.Priority.Medium,"NewUpTrend", confirmedUpTrend, rearmTime, Color.Black, upColor);
						}
						catch{}
					}
					else if(!currentUpTrend && priorUpTrend)	 
					{
						try
						{
								Alert("NewDownTrend", NinjaTrader.Cbi.Priority.Medium,"NewDownTrend", confirmedDownTrend, rearmTime, Color.Black, downColor);
						}
						catch{}
					}
				}				
				
			}
			
			if(ShowPaintBars && candles) 
			{
				if( Open[0] <  Close[0])
					BarColor = Color.FromArgb(alpha, CandleOutlineColor);
				else
					BarColor = CandleOutlineColor;
			}	
			
			if (soundAlert && !Historical && IsConnected() && !CalculateOnBarClose)
			{
				if (Close[0] > currentStopShort && !currentUpTrend)
				{
					try
					{
							Alert("PotentialUpTrend", NinjaTrader.Cbi.Priority.Medium,"PotentialUpTrend", potentialUpTrend, rearmTime, Color.Black, upColor);
					}
					catch{}
				}
				else if (Close[0] < currentStopLong && currentUpTrend)
				{
					try
					{
							Alert("PotentialDownTrend", NinjaTrader.Cbi.Priority.Medium,"PotentialDownTrend", potentialDownTrend, rearmTime, Color.Black, downColor);
					}
					catch{}
				}
			}	
		}

        #region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries StopDot
		{
			get { return Values[0]; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore]
		public DataSeries StopLine
		{
			get { return Values[1]; }
		}

		[Browsable(false)]
        [XmlIgnore()]		
        public BoolSeries UpTrend
        {
            get { return upTrend; }
        }

		[Description("Median period")]
        [GridCategory("Parameters")]
		[Gui.Design.DisplayName("MA period")]
        public int Period
        {
            get { return period; }
            set { period = Math.Max(1, value); }
        }

		/// <summary>
		/// </summary>
		[Description("Moving average type for baseline")]
		[GridCategory("Parameters")]
		[Gui.Design.DisplayNameAttribute("Baseline smoothing")]
		public anaHiLoActivatorBaseTypeU1 ThisBaseType
		{
			get { return thisBaseType; }
			set { thisBaseType = value; }
		}

		[Description("Show arrows when trendline is violated?")]
        [Category("Display and Sound Options")]
		[Gui.Design.DisplayName ("Show arrows")]
        public bool ShowArrows
        {
            get { return showArrows; }
            set { showArrows = value; }
        }

		[Description("Color the bars in the direction of the trend?")]
        [Category("Display and Sound Options")]
		[Gui.Design.DisplayName ("Show paintbars")]
        public bool ShowPaintBars
        {
            get { return showPaintBars; }
            set { showPaintBars = value; }
        }

		[Description("Show stop line")]
        [Category("Display and Sound Options")]
		[Gui.Design.DisplayName ("Show stop line")]
        public bool ShowStopLine
        {
            get { return showStopLine; }
            set { showStopLine = value; }
        }

		[Description("Sound alerts activated")]
        [Category("Display and Sound Options")]
		[Gui.Design.DisplayName("Sound alert active")]
        public bool SoundAlert
        {
            get { return soundAlert; }
            set { soundAlert = value; }
        }
		
		[Description("Allows the indicator to paint back a single tick on realtime data with setting CalculateOnBarClose = false. This will be automatically ignored if the indicator is called from a strategy")]
        [Category("Display and Sound Options")]
		[Gui.Design.DisplayName("One-tick repaint")]
        public bool OneTickRepaint
        {
            get { return oneTickRepaint; }
            set { oneTickRepaint = value; }
        }
		
		/// <summary>
		/// </summary>
        [XmlIgnore()]		
		[Description("Select color for uptrend")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Uptrend")]
		public Color UpColor
		{
			get { return upColor; }
			set { upColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string UpColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(upColor); }
			set { upColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
        [XmlIgnore()]		
		[Description("Select color for downtrend")]
		[Category("Plot Colors")]
		[Gui.Design.DisplayName("Downtrend")]
		public Color DownColor
		{
			get { return downColor; }
			set { downColor = value; }
		}
		
		// Serialize Color object
		[Browsable(false)]
		public string DownColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(downColor); }
			set { downColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Width for stop dots.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Width stop dots")]
		public int Plot0Width
		{
			get { return plot0Width; }
			set { plot0Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for stop dots.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style stop dots")]
		public PlotStyle Plot0Style
		{
			get { return plot0Style; }
			set { plot0Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for stop dots.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style stop dots")]
		public DashStyle Dash0Style
		{
			get { return dash0Style; }
			set { dash0Style = value; }
		} 
		
		/// <summary>
		/// </summary>
		[Description("Width for stop line.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Width stop line")]
		public int Plot1Width
		{
			get { return plot1Width; }
			set { plot1Width = Math.Max(1, value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("PlotStyle for stop line.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Plot style stop line")]
		public PlotStyle Plot1Style
		{
			get { return plot1Style; }
			set { plot1Style = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("DashStyle for stop line.")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Dash style stop line")]
		public DashStyle Dash1Style
		{
			get { return dash1Style; }
			set { dash1Style = value; }
		} 

		[Description("When paint bars are activated, this parameter sets the opacity of the upclose bars")]
		[Category("Plot Parameters")]
		[Gui.Design.DisplayNameAttribute("Upclose opacity")]
		public int Opacity
		{
		get { return opacity; }
		set { opacity = value; }
		}
		
		[Description("Sound file for confirmed new uptrend")]
		[Category("Sound Files")]
		[Gui.Design.DisplayNameAttribute("New uptrend")]
		public string ConfirmedUpTrend
		{
			get { return confirmedUpTrend; }
			set { confirmedUpTrend = value; }
		}
		
		[Description("Sound file for confirmed new downtrend")]
		[Category("Sound Files")]
		[Gui.Design.DisplayNameAttribute("New downtrend")]
		public string ConfirmedDownTrend
		{
			get { return confirmedDownTrend; }
			set { confirmedDownTrend = value; }
		}
		
		[Description("Sound file for potential new uptrend")]
		[Category("Sound Files")]
		[Gui.Design.DisplayNameAttribute("Potential uptrend")]
		public string PotentialUpTrend
		{
			get { return potentialUpTrend; }
			set { potentialUpTrend = value; }
		}
		
		[Description("Sound file for potential new downtrend")]
		[Category("Sound Files")]
		[Gui.Design.DisplayNameAttribute("Potential downtrend")]
		public string PotentialDownTrend
		{
			get { return potentialDownTrend; }
			set { potentialDownTrend = value; }
		}
		
		[Description("Rearm time for alert in seconds")]
		[Category("Sound Alerts")]
		[Gui.Design.DisplayNameAttribute("Rearm time (sec)")]
		public int RearmTime
		{
		get { return rearmTime; }
		set { rearmTime = value; }
		}
		#endregion
		
		#region Miscellaneous
		private bool IsConnected()
        {
			if ( Bars.MarketData != null
					&& Bars.MarketData.Connection.PriceStatus == Cbi.ConnectionStatus.Connected
					&& Bars.Session.InSession(Now, Bars.Period, true))
				return true;
			else
            	return false;
        }		
		
		private DateTime Now
		{
			get 
			{ 
				DateTime now = (Bars.MarketData.Connection.Options.Provider == Cbi.Provider.Replay ? Bars.MarketData.Connection.Now : DateTime.Now); 

				if (now.Millisecond > 0)
					now = Cbi.Globals.MinDate.AddSeconds((long) System.Math.Floor(now.Subtract(Cbi.Globals.MinDate).TotalSeconds));

				return now;
			}
		}		
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private anaHiLoActivatorU1[] cacheanaHiLoActivatorU1 = null;

        private static anaHiLoActivatorU1 checkanaHiLoActivatorU1 = new anaHiLoActivatorU1();

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public anaHiLoActivatorU1 anaHiLoActivatorU1(int period, anaHiLoActivatorBaseTypeU1 thisBaseType)
        {
            return anaHiLoActivatorU1(Input, period, thisBaseType);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public anaHiLoActivatorU1 anaHiLoActivatorU1(Data.IDataSeries input, int period, anaHiLoActivatorBaseTypeU1 thisBaseType)
        {
            if (cacheanaHiLoActivatorU1 != null)
                for (int idx = 0; idx < cacheanaHiLoActivatorU1.Length; idx++)
                    if (cacheanaHiLoActivatorU1[idx].Period == period && cacheanaHiLoActivatorU1[idx].ThisBaseType == thisBaseType && cacheanaHiLoActivatorU1[idx].EqualsInput(input))
                        return cacheanaHiLoActivatorU1[idx];

            lock (checkanaHiLoActivatorU1)
            {
                checkanaHiLoActivatorU1.Period = period;
                period = checkanaHiLoActivatorU1.Period;
                checkanaHiLoActivatorU1.ThisBaseType = thisBaseType;
                thisBaseType = checkanaHiLoActivatorU1.ThisBaseType;

                if (cacheanaHiLoActivatorU1 != null)
                    for (int idx = 0; idx < cacheanaHiLoActivatorU1.Length; idx++)
                        if (cacheanaHiLoActivatorU1[idx].Period == period && cacheanaHiLoActivatorU1[idx].ThisBaseType == thisBaseType && cacheanaHiLoActivatorU1[idx].EqualsInput(input))
                            return cacheanaHiLoActivatorU1[idx];

                anaHiLoActivatorU1 indicator = new anaHiLoActivatorU1();
                indicator.BarsRequired = BarsRequired;
                indicator.CalculateOnBarClose = CalculateOnBarClose;
#if NT7
                indicator.ForceMaximumBarsLookBack256 = ForceMaximumBarsLookBack256;
                indicator.MaximumBarsLookBack = MaximumBarsLookBack;
#endif
                indicator.Input = input;
                indicator.Period = period;
                indicator.ThisBaseType = thisBaseType;
                Indicators.Add(indicator);
                indicator.SetUp();

                anaHiLoActivatorU1[] tmp = new anaHiLoActivatorU1[cacheanaHiLoActivatorU1 == null ? 1 : cacheanaHiLoActivatorU1.Length + 1];
                if (cacheanaHiLoActivatorU1 != null)
                    cacheanaHiLoActivatorU1.CopyTo(tmp, 0);
                tmp[tmp.Length - 1] = indicator;
                cacheanaHiLoActivatorU1 = tmp;
                return indicator;
            }
        }
    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaHiLoActivatorU1 anaHiLoActivatorU1(int period, anaHiLoActivatorBaseTypeU1 thisBaseType)
        {
            return _indicator.anaHiLoActivatorU1(Input, period, thisBaseType);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.anaHiLoActivatorU1 anaHiLoActivatorU1(Data.IDataSeries input, int period, anaHiLoActivatorBaseTypeU1 thisBaseType)
        {
            return _indicator.anaHiLoActivatorU1(input, period, thisBaseType);
        }
    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.anaHiLoActivatorU1 anaHiLoActivatorU1(int period, anaHiLoActivatorBaseTypeU1 thisBaseType)
        {
            return _indicator.anaHiLoActivatorU1(Input, period, thisBaseType);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public Indicator.anaHiLoActivatorU1 anaHiLoActivatorU1(Data.IDataSeries input, int period, anaHiLoActivatorBaseTypeU1 thisBaseType)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.anaHiLoActivatorU1(input, period, thisBaseType);
        }
    }
}
#endregion
